#include <SoftwareSerial.h>

char MESSAGE[300];
String phone = "xxxxxxxxx";

SoftwareSerial mySerial(5,6); // RX, TX ( TX of module -> 10 of Arduino) -- ( RX of module -> 11 of Arduino) -- (GND common)

void(* resetFunc) (void) = 0;
void GPS_data();
void check_GPS();
void sendSMS();

char frame[256];

byte GNSSrunstatus;  //<<<<<<< was char array
byte Fixstatus;           //<<<<<<< was char array
char UTCdatetime[15];

char latitude[10];
float latitude_val;

char longitude[11];
float longitude_val;

long lat_val;
long lon_val;

long lat_high;
long lat_low;

long lon_high;
long lon_low;

int lat_check;
int lon_check;

char altitude[8];
char speedOTG[6];
char course[6];
char fixmode[1];
char HDOP[4];
char PDOP[4];
char VDOP[4];
char satellitesinview[2];
char GNSSsatellitesused[2];
char GLONASSsatellitesused[2];
char cn0max[2];
char HPA[6];
char VPA[6];



bool inRange(long val, long minimum, long  maximum)
{
  return ((minimum <= val) && (val <= maximum));
}

void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);
  mySerial.begin(9600);
  //pinMode(13, OUTPUT);

   mySerial.println("AT\r"); //Handshaking with SIM808
  updateSerial();
  mySerial.println("AT+CSQ"); //Signal quality test, value range is 0-31 , 31 is the best
  updateSerial();
  mySerial.println("AT+CCID"); //Read SIM information to confirm whether the SIM is plugged
  updateSerial();
  mySerial.println("AT+CPIN?"); //Check whether it has registered in the network
  updateSerial();
  GPS_data(); //Initial GPS Data
  
  lat_high = lat_val + 200;
  lat_low = lat_val - 200;
  lon_high = lon_val + 200;
  lon_low = lon_val - 200;
  
  /*
  lat_high = latitude_val + 0.00005;
  lat_low = latitude_val - 0.00005;
  lon_high = longitude_val + 0.00005;
  lon_low = longitude_val - 0.00005;
  */
}

void loop() {
  updateSerial();
  GPS_data();  
}

void updateSerial()
{
  delay(500);
  while (Serial.available()) 
  {
    mySerial.write(Serial.read());//Forward what Serial received to Software Serial Port
  }
  while(mySerial.available()) 
  {
    Serial.write(mySerial.read());//Forward what Software Serial received to Serial Port
  }
}

void GPS_data(){
  mySerial.println("AT+CGNSPWR=1");
  updateSerial();
  mySerial.println("AT+CGNSSEQ=\"RMC\"");
  updateSerial();
  mySerial.println("AT+CGNSINF");
  check_GPS();
}

void check_GPS(){

  int8_t counter, answer;
  long previous;

  counter = 0;
  answer = 0;
  memset(frame, '\0', sizeof(frame));    // Initialize the string
  previous = millis();

  do {

    if (mySerial.available() != 0) {
      frame[counter] = mySerial.read();
      counter++;

      if (strstr(frame, "OK") != NULL)
      {
        answer = 1;
      }
    }
    // Waits for the asnwer with time out
  }
  while ((answer == 0) && ((millis() - previous) < 2000));
  
  frame[counter - 3] = '\0';

  // Parses the string
  strtok(frame, ": ");
  GNSSrunstatus = atoi(strtok(NULL, ","));//Gets GNSS run status
  Fixstatus = atoi(strtok(NULL, ",")); // Gets Fix status
  strcpy(UTCdatetime, strtok(NULL, ".")); // Gets UTC date & time
  strtok(NULL,",");  // skip three zeroes
  strcpy(latitude, strtok(NULL, ",")); // 
  strcpy(longitude, strtok(NULL, ",")); // Gets longitude
  strcpy(altitude, strtok(NULL, ",")); // Gets MSL altitude


    if (Fixstatus != 0){

      Serial.println("GPS Data received");
      
      Serial.print("Runstatus: ");
      Serial.println(GNSSrunstatus);
      Serial.print("Fixstatus: ");
      Serial.println(Fixstatus);
      Serial.print("UTCdatetime: ");
      Serial.println(UTCdatetime);
      Serial.print("latitude: ");
      Serial.println(latitude);
      Serial.print("longitude: ");
      Serial.println(longitude);
      Serial.print("altitude: ");
      Serial.println(altitude);

      //Serial.println(lat_high);
      latitude_val = atof(latitude);
      
      lat_val = latitude_val*1000000;  //13.000000 -> 13000000
      
      Serial.println(lat_val); 
      
      longitude_val = atof(longitude);
      lon_val = longitude_val*1000000;
      
      Serial.println(lat_high);
      
      if (lat_high == 0){
        sprintf(MESSAGE, "Latitude : %s\nLongitude : %s\nAltitude : %s km\n\nInitial Position of the Module\n\nhttp://maps.google.com/maps?q=%s,%s\n", latitude, longitude, altitude, latitude, longitude);
        sendSMS();
        
      }
      else{

        lat_check = inRange(lat_val, lat_high, lat_low);
        lon_check = inRange(lon_val, lon_high, lon_low);
        Serial.print(lat_check);
        Serial.println(lon_check);
        if (!(lat_check)){
  
          sprintf(MESSAGE, "Latitude : %s\nLongitude : %s\nAltitude : %s km\n\nALERT!! Module is outside the GeoFence\n\nhttp://maps.google.com/maps?q=%s,%s\n", latitude, longitude, altitude, latitude, longitude);
          sendSMS();
        }
      
      }

    }
     else if ((Fixstatus == 0) && (lat_high == 0)){
      Serial.println("Resetting Arduino to check GPS data");
      delay(5000);
      resetFunc();
    }
}

void sendSMS()
{
  Serial.println("Start to send message ...");
  
  Serial.println(MESSAGE);
  Serial.println(phone);

  mySerial.println("AT+CSMP=17,167,0,0"); //Handshaking with SIM808
  updateSerial();
  mySerial.println("AT+CMGF=1"); //Handshaking with SIM808
  updateSerial();
  String sim_sms = "AT+CMGS=\"" + phone + "\"\r";
  mySerial.println(sim_sms); //Handshaking with SIM808
  updateSerial();
  mySerial.print(MESSAGE);       //This is the text to send to the phone number, don't make it too long or you have to modify the SoftwareSerial buffer
  delay(500);
  mySerial.print((char)26);// (required according to the datasheet)
  Serial.println("Check SMS");
}